// widget/jsfx/dynaFaces.js
//
// The contents of this file are subject to the terms
// of the Common Development and Distribution License
// (the License).  You may not use this file except in
// compliance with the License.
// 
// You can obtain a copy of the license at
// https://woodstock.dev.java.net/public/CDDLv1.0.html.
// See the License for the specific language governing
// permissions and limitations under the License.
// 
// When distributing Covered Code, include this CDDL
// Header Notice in each file and include the License file
// at https://woodstock.dev.java.net/public/CDDLv1.0.html.
// If applicable, add the following below the CDDL Header,
// with the fields enclosed by brackets [] replaced by
// you own identifying information:
// "Portions Copyrighted [year] [name of copyright owner]"
// 
// Copyright 2007 Sun Microsystems, Inc. All rights reserved.
//

webui.suntheme.dojo.provide("webui.suntheme.widget.jsfx.dynaFaces");

webui.suntheme.dojo.require("webui.suntheme.browser");
webui.suntheme.dojo.require("webui.suntheme.theme.common");
webui.suntheme.dojo.require("webui.suntheme.widget.eventBase");

/**
 * @class This class contains functions to obtain JSF Extensions resources.
 * @static
 * @private
 */
webui.suntheme.widget.jsfx.dynaFaces = {
    /**
     * This function is used to initialize the environment with Object literals.
     *
     * @param {Object} props Key-Value pairs of properties.
     * @config (Object) ajax Key-Value pairs of Ajax properties.
     * @return {boolean} true if successful; otherwise, false.
     * @private
     */
    init: function(props) {
        if (props == null) {
            return false;
        }
        
        // Load JSF Extensions immediately.
        if (props.ajax.webuiJsfx == true) {
            webui.suntheme.widget.jsfx.dynaFaces.loadJsfx();
        } else {
            // Override default publish functionality to lazily load JSFX.
            webui.suntheme.widget.eventBase.prototype._publish = 
                function(topic, props) {
                    // Load JSF Extensions and publish event via a callback, 
                    // ensuring all resources have been loaded.
                    webui.suntheme.widget.jsfx.dynaFaces.loadJsfx(function() {

                    // Publish an event for custom AJAX implementations to listen for.
                    webui.suntheme.dojo.publish(topic, props);
                    return true;
                });
            }
        }
        return true;
    },

    /**
     * Load JSF Extensions.
     *
     * @param {Function} callback The JavaScript function to invoke on load.
     * @return {boolean} true if successful; otherwise, false.
     * @private
     */
    loadJsfx: function(callback) {
        if (typeof DynaFaces != "undefined") {
            return callback();
        }

        // Get script URLs.
        var common = webui.suntheme.theme.common;
        var pUrl = common.getJavaScript((webui.suntheme.config.isDebug == true)
            ? "prototypeUncompressed" : "prototype");
        var jUrl = common.getJavaScript((webui.suntheme.config.isDebug == true)
            ? "jsfxUncompressed" : "jsfx");

        // Ensure Prototype is loaded first using a callback.
        var jsfxCallback = function() {
            webui.suntheme.widget.jsfx.dynaFaces.loadScript(jUrl, callback);
        };

        // Load Prototype and DynaFaces.
        if (typeof Prototype == "undefined") {
            webui.suntheme.widget.jsfx.dynaFaces.loadScript(pUrl, jsfxCallback);
        } else {
            jsfxCallback();
        }
        return true;
    },

    /**
     * Load JavaScript via script tag.
     *
     * Note: Prototype must be added to the global scope. Therefore,
     * dojo._loadPath() and/or dojo._loadUri() will not work here.
     *
     * @param {String} url The script url to load.
     * @param {Function} callback The JavaScript function to invoke on load.
     * @return {boolean} true if successful; otherwise, false.
     * @private
     */
    loadScript: function(url, callback) {
        if (url == null) {
            console.debug("Error: Cannot load " + url);
            return false;
        }

        // Get HTML head tag for use as parent node.
        var domNode = document.getElementsByTagName("head")[0];
        if (domNode == null) {
            console.debug("Error: No HTML head tag found.");
            return false;
        }

        // Create script tag.
        var script = document.createElement('script');
        script.type = 'text/javascript';
        script.src = url;
        if (callback) {
            if (webui.suntheme.browser.isIe()) {
                script.onreadystatechange = function () {
                    // IE 7 won't return 'complete', but 'loaded' seems to work.
                    if (script.readyState == 'loaded' 
                            || script.readyState == 'complete') {
                        callback();
                    }
                }
            } else {
                script.onload = callback;
            }
        }
        domNode.appendChild(script);
        return true;
    }
}

// Initialize the environment.
webui.suntheme.widget.jsfx.dynaFaces.init(webui.suntheme.config);
